@@ -240,6 +240,11 @@ LockStruct  LockProcs[] =
         {"juggle",
          10000, 200, 1000, 1, 64, 1.0, "",
          "Shows a Juggler, juggling", 0, NULL, NULL},
+#ifdef USE_GL
+	{"juggler3d",
+         10000, 200, 1000, 1, 64, 1.0, "",
+	 "Shows a 3D Juggler, juggling", 0, NULL, "#ifdef USE_GL"},
+#endif
 	{"julia",
 	 10000, 1000, 20, 1, 64, 1.0, "",
 	 "Shows the Julia set", 0, NULL, NULL},
@@ -383,10 +388,10 @@ LockStruct  LockProcs[] =
          15000, 1, 2, 1, 64, 1.0, "",
          "Shows GL's Sierpinski gasket", 0, NULL, "#ifdef USE_GL"},
 #endif
-#if defined(USE_GL) && defined( USE_UNSTABLE )
+#ifdef USE_GL
 	{"skewb",
 	 100000, -30, 5, 1, 64, 1.0, "",
-	 "Shows an auto-solving Skewb", 0, NULL, "#if defined(USE_GL) && defined( USE_UNSTABLE )"},
+	 "Shows an auto-solving Skewb", 0, NULL, "#ifdef USE_GL"},
 #endif
 	{"slip",
 	 50000, 35, 50, 1, 64, 1.0, "",
