@@ -49,6 +49,7 @@ static const char sccsid[] = "@(#)ant.c	5.24 2007/01/18 xlockmore";
   Ants    Square   4 (or 8)
   Bees    Hexagon  6
   Bees    Triangle 3 (or 9, 12)
+          Pentagon 5 (or 7)
 
   Neighbors 6 and neighbors 3 produce the same Turk ants.
 */
@@ -81,7 +82,7 @@ static const char sccsid[] = "@(#)ant.c	5.24 2007/01/18 xlockmore";
 #define FONT_WIDTH 15
 
 /*-
- * neighbors of 0 randomizes it for 3, 4, 6, 8, 12 (last 2 are less likely)
+ * neighbors of 0 randomizes it for 3, 4, 5, 6, 7, 8, 12 (last 3 are less likely)
  */
 
 #define DEF_NEIGHBORS  "0"      /* choose random value */
@@ -160,7 +161,7 @@ const ModStruct ant_description =
 #define MINGRIDSIZE 24
 #define MINSIZE 1
 #define MINRANDOMSIZE 5
-#define ANGLES 360
+#define ANGLES(x) ((x==7) ? 357 : 360)  /* 360 not divisible by 7 */
 
 typedef struct {
 	unsigned char color;
@@ -176,7 +177,7 @@ typedef struct {
 
 typedef struct {
 	Bool        painted, vertical;
-	int         neighbors;
+	int         neighbors, polygon;
 	int         generation;
 	int         xs, ys;
 	int         xb, yb;
@@ -199,21 +200,22 @@ typedef struct {
 	Pixmap      pixmaps[NUMSTIPPLES - 1];
 	int         labelOffsetX, labelOffsetY;
 	char        ruleString[50];
-	union {
-		XPoint      hexagon[7];		/* Need more than 6 for truchet */
-		XPoint      triangle[2][4];	/* Need more than 3 for truchet */
+	union {		/* Need extra point for truchet */
+		XPoint      hexagon[7];
+		XPoint      triangle[2][4];
+		XPoint      pentagon[4][6];
 	} shape;
 } antfarmstruct;
 
 static char plots[] =
-{3, 4, 6, 8,
+{3, 4, 5, 6, 7, 8,
 #ifdef NUMBER_9
  9,
 #endif
  12};
 
 #define NEIGHBORKINDS ((long) (sizeof plots / sizeof *plots))
-#define GOODNEIGHBORKINDS 3
+#define GOODNEIGHBORKINDS 4
 
 /* Relative ant moves */
 #define FS 0			/* Step */
@@ -276,7 +278,7 @@ static unsigned char tables[][3 * NUMSTIPPLES * STATES + 2] =
 #define NTABLES   (sizeof tables / sizeof tables[0])
 
 static void
-position_of_neighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
+positionOfNeighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
 {
 	int col = *pcol, row = *prow;
 
@@ -333,6 +335,153 @@ position_of_neighbor(antfarmstruct * ap, int dir, int *pcol, int *prow)
 		default:
 			(void) fprintf(stderr, "wrong direction %d\n", dir);
 		}
+	} else if (ap->neighbors == 5 || ap->neighbors == 7) {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= ap->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= ap->ncols) ? 1 : col + 3;
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? ap->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? ap->ncols - 2 : col - 3;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? ap->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? ap->ncols - 1 : col - 1;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				row = (row + 1 == ap->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= ap->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= ap->ncols) ? 0 : col + 1;
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? ap->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
 	} else {		/* TRI */
 		if ((col + row) % 2) {	/* right */
 			switch (dir) {
@@ -494,7 +643,7 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 	Window window = MI_WINDOW(mi);
 
 	if (ap->neighbors == 6) {
-		int         ccol = 2 * col + !(row & 1), crow = 2 * row;
+		int ccol = 2 * col + !(row & 1), crow = 2 * row;
 
 		if (ap->vertical) {
 			ap->shape.hexagon[0].x = ap->xb + ccol * ap->xs;
@@ -505,7 +654,7 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		}
 		if (ap->xs == 1 && ap->ys == 1)
 			XDrawPoint(display, window, gc,
-			ap->shape.hexagon[0].x, ap->shape.hexagon[0].y);
+				ap->shape.hexagon[0].x, ap->shape.hexagon[0].y);
 		else
 			XFillPolygon(display, window, gc,
 			    ap->shape.hexagon, 6, Convex, CoordModePrevious);
@@ -513,6 +662,57 @@ fillcell(ModeInfo * mi, GC gc, int col, int row)
 		XFillRectangle(display, window, gc,
 		ap->xb + ap->xs * col, ap->yb + ap->ys * row,
 	 	ap->xs - (ap->xs > 3), ap->ys - (ap->ys > 3));
+	} else if (ap->neighbors == 5 || ap->neighbors == 7) {
+		int map[4] = {2, 0, 1, 3};
+		int orient = ((row & 1) * 2 + col) % 4;
+		int offsetX = 0, offsetY = 0;
+		switch (orient) {
+#if 0
+		case 0: /* up */
+			offsetX = ap->xs;
+			break;
+		case 1: /* down */
+			offsetY = 4 * ap->ys / 3;
+			break;
+		case 2: /* left */
+			offsetX = -ap->xs / 3;
+			offsetY = 2 * ap->ys / 3;
+			break;
+		case 3: /* right */
+			offsetX = 4 * ap->xs / 3;
+			offsetY = 2 * ap->ys / 3;
+			break;
+#else
+		case 0: /* up */
+			offsetX = ap->xs;
+			offsetY = -1;
+			break;
+		case 1: /* down */
+			offsetY = ap->ys;
+			break;
+		case 2: /* left */
+			offsetX = -2;
+			offsetY = ap->ys / 2;
+			break;
+		case 3: /* right */
+			offsetX = ap->xs;
+			offsetY = ap->ys / 2;
+			break;
+#endif
+		default:
+			(void) printf("wrong orient %d\n", orient);
+		}
+		orient = map[orient];
+		ap->shape.pentagon[orient][0].x =  ap->xb +
+			col * ap->xs + offsetX;
+		ap->shape.pentagon[orient][0].y = ap->yb +
+			row * ap->ys + offsetY;
+		if (ap->xs <= 2 || ap->ys <= 2)
+			XDrawPoint(display, window, gc,
+				ap->shape.pentagon[orient][0].x, ap->shape.pentagon[orient][0].y);
+		else
+			XFillPolygon(display, window, gc,
+				ap->shape.pentagon[orient], 5, Convex, CoordModePrevious);
 	} else {		/* TRI */
 		int orient = (col + row) % 2;	/* O left 1 right */
 		Bool small = (ap->xs <= 3 || ap->ys <= 3);
@@ -787,7 +987,7 @@ draw_anant(ModeInfo * mi, int direction, int col, int row)
 				return;
 			hex.x = ap->xb + ccol * ap->xs;
 			hex.y = ap->yb + crow * ap->ys + ap->ys / 2;
-			ang = direction * ap->neighbors / ANGLES;
+			ang = direction * ap->neighbors / ANGLES(ap->neighbors);
 			for (side = 0; side < ap->neighbors; side++) {
 				if (side) {
 					hex.x -= ap->shape.hexagon[side].x / 2;
@@ -882,7 +1082,7 @@ draw_anant(ModeInfo * mi, int direction, int col, int row)
 				tri.x += (ap->xs / 6 - 1);
 			else
 				tri.x -= (ap->xs / 6 - 1);
-			ang = direction * ap->neighbors / ANGLES;
+			ang = direction * ap->neighbors / ANGLES(ap->neighbors);
 			/* approx... does not work that well for even numbers */
 			if (
 #ifdef NUMBER_9
@@ -938,32 +1138,34 @@ RandomSoup(mi)
 static short
 fromTableDirection(unsigned char dir, int local_neighbors)
 {
+	int angles = ANGLES(local_neighbors);
+
 	/* Crafted to work for odd number of neighbors */
 	switch (dir) {
 		case FS:
 			return 0;
 		case TLS:
-			return (ANGLES / local_neighbors);
+			return (angles / local_neighbors);
 		case THLS:
-			return (2 * ANGLES / local_neighbors);
+			return (2 * angles / local_neighbors);
 		case TBS:
-			return ((local_neighbors / 2) * ANGLES / local_neighbors);
+			return ((local_neighbors / 2) * angles / local_neighbors);
 		case THRS:
-			return (ANGLES - 2 * ANGLES / local_neighbors);
+			return (angles - 2 * angles / local_neighbors);
 		case TRS:
-			return (ANGLES - ANGLES / local_neighbors);
+			return (angles - angles / local_neighbors);
 		case SF:
-			return ANGLES;
+			return angles;
 		case STL:
-			return (ANGLES + ANGLES / local_neighbors);
+			return (angles + angles / local_neighbors);
 		case STHL:
-			return (ANGLES + 2 * ANGLES / local_neighbors);
+			return (angles + 2 * angles / local_neighbors);
 		case STB:
-			return (ANGLES + (local_neighbors / 2) * ANGLES / local_neighbors);
+			return (angles + (local_neighbors / 2) * angles / local_neighbors);
 		case STHR:
-			return (2 * ANGLES - 2 * ANGLES / local_neighbors);
+			return (2 * angles - 2 * angles / local_neighbors);
 		case STR:
-			return (2 * ANGLES - ANGLES / local_neighbors);
+			return (2 * angles - angles / local_neighbors);
 		default:
 			(void) fprintf(stderr, "wrong direction %d from table\n", dir);
 	}
@@ -1229,6 +1431,7 @@ init_ant(ModeInfo * mi)
 	if (ap->neighbors == 6) {
 		int         nccols, ncrows;
 
+		ap->polygon = 6;
 		if (!ap->vertical) {
 			ap->height = MI_WIDTH(mi);
 			ap->width = MI_HEIGHT(mi);
@@ -1277,6 +1480,7 @@ init_ant(ModeInfo * mi)
 		ap->shape.hexagon[6].x = 0;
 		ap->shape.hexagon[6].y = 0;
 	} else if (ap->neighbors == 4 || ap->neighbors == 8) {
+		ap->polygon = 4;
 		if (size < -MINSIZE) {
 			ap->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -1296,9 +1500,53 @@ init_ant(ModeInfo * mi)
 		ap->nrows = MAX(ap->height / ap->ys, 2);
 		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
 		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
+	} else if (ap->neighbors == 5 || ap->neighbors == 7) {
+		int orient, sides;
+		ap->polygon = 5;
+		if (ap->width < 2)
+			ap->width = 2;
+		if (ap->height < 2)
+			ap->height = 2;
+		if (size < -MINSIZE)
+			ap->xs = NRAND(MIN(-size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+				MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size) {
+				int min = MIN(ap->width, ap->height) / (4 * MINGRIDSIZE);
+				int max = MIN(ap->width, ap->height) / (MINGRIDSIZE);
+
+				ap->xs = MAX(MINSIZE, min + NRAND(max - min + 1));
+			} else
+				ap->xs = MINSIZE;
+		} else
+			ap->xs = MIN(size, MAX(MINSIZE, MIN(ap->width, ap->height) /
+				MINGRIDSIZE));
+		ap->ys = ap->xs * 2;
+		ap->ncols = (MAX(ap->width / ap->xs, 8) / 4) * 4;
+		ap->nrows = (MAX(ap->height / ap->ys, 8)) / 2 * 2;
+		ap->xb = (ap->width - ap->xs * ap->ncols) / 2;
+		ap->yb = (ap->height - ap->ys * ap->nrows) / 2;
+		for (orient = 0; orient < 4; orient++) {
+			for (sides = 0; sides < 5; sides++) {
+#if 0
+				ap->shape.pentagon[orient][sides].x =
+					2 * ap->xs * pentagonUnit[orient][sides].x / 3;
+				ap->shape.pentagon[orient][sides].y =
+					ap->ys * pentagonUnit[orient][sides].y / 3;
+#else
+				ap->shape.pentagon[orient][sides].x =
+					ap->xs * pentagonUnit[orient][sides].x / 2;
+				ap->shape.pentagon[orient][sides].y =
+					ap->ys * pentagonUnit[orient][sides].y / 4;
+#endif
+			}
+			ap->shape.pentagon[orient][5].x = 0;
+			ap->shape.pentagon[orient][5].y = 0;
+		}
 	} else {		/* TRI */
 		int orient;
 
+		ap->polygon = 3;
 		if (!ap->vertical) {
 			ap->height = MI_WIDTH(mi);
 			ap->width = MI_HEIGHT(mi);
@@ -1388,12 +1636,13 @@ init_ant(ModeInfo * mi)
 	col = ap->ncols / 2;
 	if (col > 0 && ((ap->neighbors % 2) || ap->neighbors == 12) && (LRAND() & 1))
 		col--;
-	dir = NRAND(ap->neighbors) * ANGLES / ap->neighbors;
+	if (ap->polygon == 5) {
+		dir = 0;
+		col += NRAND(4);
+	} else {
+		dir = NRAND(ap->polygon) * ANGLES(ap->neighbors) / ap->neighbors;
+	}
 	ap->init_dir = dir;
-#ifdef NUMBER_9
-	if (ap->neighbors == 9 && !((col + row) & 1))
-		dir = (dir + ANGLES - ANGLES / (ap->neighbors * 2)) % ANGLES;
-#endif
 	/* Have them all start in the same spot, why not? */
 	for (i = 0; i < ap->n; i++) {
 		ap->ants[i].col = col;
@@ -1434,14 +1683,14 @@ draw_ant(ModeInfo * mi)
 		/* Find direction of Bees or Ants. */
 		/* Translate relative direction to actual direction */
 		old_dir = anant->direction;
-		chg_dir = (2 * ANGLES - status->direction) % ANGLES;
-		anant->direction = (chg_dir + old_dir) % ANGLES;
+		chg_dir = (2 * ANGLES(ap->neighbors) - status->direction) % ANGLES(ap->neighbors);
+		anant->direction = (chg_dir + old_dir) % ANGLES(ap->neighbors);
 		if (ap->truchet) {
 			int         a = 0, b;
 
 			if (ap->neighbors == 6) {
 				if (ap->sharpturn) {
-					a = (((ANGLES + anant->direction - old_dir) % ANGLES) == 240);
+					a = (((ANGLES(ap->neighbors) + anant->direction - old_dir) % ANGLES(ap->neighbors)) == 240);
 	/* should be some way of getting rid of the init_dir dependency... */
 					b = !(ap->init_dir % 120);
 					a = ((a && !b) || (b && !a));
@@ -1469,11 +1718,11 @@ draw_ant(ModeInfo * mi)
 		anant->state = status->next;
 
 		/* Allow step first then turn */
-		old_dir = ((status->direction < ANGLES) ? anant->direction : old_dir);
+		old_dir = ((status->direction < ANGLES(ap->neighbors)) ? anant->direction : old_dir);
 #ifdef DEBUG
 		(void) printf("old_dir %d, col %d, row %d", old_dir, anant->col, anant->row);
 #endif
-		position_of_neighbor(ap, old_dir, &(anant->col), &(anant->row));
+		positionOfNeighbor(ap, old_dir, &(anant->col), &(anant->row));
 #ifdef DEBUG
 		(void) printf(", ncol %d, nrow %d\n", anant->col, anant->row);
 #endif
