@@ -660,7 +660,7 @@ static void heading(antfarm3dstruct * ap, short * direction, short * gravity,
 }
 
 static void
-position_of_neighbor(antfarm3dstruct * ap, int dir,
+positionOfNeighbor(antfarm3dstruct * ap, int dir,
 		int *pcol, int *prow, int *pstack)
 {
 	int col = *pcol, row = *prow, stack = *pstack;
@@ -954,7 +954,7 @@ RunAnt3D(antfarm3dstruct * ap)
 		(void) printf("state %d, col %d, row %d, stack %d",
 			anant->state, anant->col, anant->row, anant->stack);
 #endif
-		position_of_neighbor(ap, anant->direction,
+		positionOfNeighbor(ap, anant->direction,
 			&(anant->col), &(anant->row), &(anant->stack));
 		SetList3D(anant->col, anant->row, anant->stack,
 			NCOLORS);
