@@ -884,7 +884,7 @@ drawDclock(ModeInfo * mi)
 
 	XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
 	/* This could leave screen dust on the screen if the width changes
-	   But that only happens once a day...
+	   But that only happens once a day... (well maybe not for counts)
 	   ... this is solved by the ClearWindow above
 	 */
 	ERASE_IMAGE(display, window, gc,
@@ -1121,7 +1121,8 @@ init_dclock(ModeInfo * mi)
 		dp->text_ascent = mode_font->ascent;
 #endif
 		if (message && *message) {
-			char *p, *p2, *p3, buf[BUFSIZ]; int width, height, w;
+			char *p, *p2, *p3, buf[BUFSIZ];
+			int height, w;
 
 			p = strncpy(buf, message, BUFSIZ);
 			while (strlen(p) > 0) {
