@@ -46,6 +46,7 @@ static const char sccsid[] = "@(#)demon.c	5.24 2007/01/18 xlockmore";
   Square   4 or 8
   Hexagon  6
   Triangle 3, 9, or 12
+  Pentagon 5 or 7
 */
 
 #ifdef STANDALONE
@@ -71,7 +72,7 @@ static const char sccsid[] = "@(#)demon.c	5.24 2007/01/18 xlockmore";
 #ifdef MODE_demon
 
 /*-
- * neighbors of 0 randomizes it between 3, 4, 6, 8, 9, and 12.
+ * neighbors of 0 randomizes it between 3, 4, 5, 6, 7, 8, 9, and 12.
  */
 #define DEF_NEIGHBORS  "0"      /* choose random value */
 #define DEF_VERTICAL "False"
@@ -93,7 +94,7 @@ static argtype vars[] =
 };
 static OptionStruct desc[] =
 {
-	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12"},
+	{(char *) "-neighbors num", (char *) "squares 4 or 8, hexagons 6, triangles 3, 9 or 12, pentagons 5 or 7"},
 	{(char *) "-/+vertical", (char *) "change orientation for hexagons and triangles"}
 };
 
@@ -118,7 +119,7 @@ ModStruct   demon_description =
 #define MINSTATES 2
 #define MINGRIDSIZE 24
 #define MINSIZE 4
-#define NEIGHBORKINDS 6
+#define NEIGHBORKINDS 8
 
 /* Singly linked list */
 typedef struct _CellList {
@@ -139,26 +140,27 @@ typedef struct {
 	int        *ncells;
 	CellList  **cellList;
 	unsigned char *oldcell, *newcell;
-	int         neighbors;
+	int         neighbors, polygon;
 	int         init_bits;
 	GC          stippledGC;
 	Pixmap      pixmaps[NUMSTIPPLES - 1];
 	union {
 		XPoint      hexagon[7];
 		XPoint      triangle[2][4];
+		XPoint      pentagon[4][6];
 	} shape;
 } demonstruct;
 
 static char plots[2][NEIGHBORKINDS] =
 {
-	{3, 4, 6, 8, 9, 12},	/* Neighborhoods */
-	{12, 16, 18, 20, 22, 24}	/* Number of states */
+	{3, 4, 5, 6, 7, 8, 9, 12},	/* Neighborhoods */
+	{12, 16, 17, 18, 19, 20, 22, 24}	/* Number of states */
 };
 
 static demonstruct *demons = (demonstruct *) NULL;
 
 static void
-drawcell(ModeInfo * mi, int col, int row, unsigned char state)
+drawCell(ModeInfo * mi, int col, int row, unsigned char state)
 {
 	demonstruct *dp = &demons[MI_SCREEN(mi)];
 	Display *display = MI_DISPLAY(mi);
@@ -205,6 +207,58 @@ drawcell(ModeInfo * mi, int col, int row, unsigned char state)
 		XFillRectangle(display, window, gc,
 			dp->xb + dp->xs * col, dp->yb + dp->ys * row,
 			dp->xs - (dp->xs > 3), dp->ys - (dp->ys > 3));
+	} else if (dp->neighbors == 5 || dp->neighbors == 7) {
+		int map[4] = {2, 0, 1, 3};
+		int orient = ((row & 1) * 2 + col) % 4;
+		int offsetX = 0, offsetY = 0;
+		switch (orient) {
+#if 0
+		case 0: /* up */
+			offsetX = dp->xs;
+			break;
+		case 1: /* down */
+			offsetY = 4 * dp->ys / 3;
+			break;
+		case 2: /* left */
+			offsetX = -dp->xs / 3;
+			offsetY = 2 * dp->ys / 3;
+			break;
+		case 3: /* right */
+			offsetX = 4 * dp->xs / 3;
+			offsetY = 2 * dp->ys / 3;
+			break;
+#else
+		case 0: /* up */
+			offsetX = dp->xs;
+			offsetY = -1;
+			break;
+		case 1: /* down */
+			offsetY = dp->ys;
+			break;
+		case 2: /* left */
+			offsetX = -2;
+			offsetY = dp->ys / 2;
+			break;
+		case 3: /* right */
+			offsetX = dp->xs;
+			offsetY = dp->ys / 2;
+			break;
+#endif
+		default:
+			(void) printf("wrong orient %d\n", orient);
+		}
+		orient = map[orient];
+		dp->shape.pentagon[orient][0].x =  dp->xb +
+			col * dp->xs + offsetX;
+		dp->shape.pentagon[orient][0].y = dp->yb +
+			row * dp->ys + offsetY;
+		if (dp->xs <= 2 || dp->ys <= 2)
+			XDrawPoint(display, window, gc,
+				dp->shape.pentagon[orient][0].x,
+				dp->shape.pentagon[orient][0].y);
+		else
+			XFillPolygon(display, window, gc,
+				dp->shape.pentagon[orient], 5, Convex, CoordModePrevious);
 	} else {		/* TRI */
 		int orient = (col + row) % 2;	/* O left 1 right */
 		Bool small = (dp->xs <= 3 || dp->ys <= 3);
@@ -376,11 +430,11 @@ draw_state(ModeInfo * mi, int state)
 	if (dp->neighbors == 6) {	/* Draw right away, slow */
 		current = dp->cellList[state];
 		while (current) {
-			int         col, row, ccol, crow;
+			int col = current->pt.x;
+			int row = current->pt.y;
+			int ccol = 2 * col + !(row & 1);
+			int crow = 2 * row;
 
-			col = current->pt.x;
-			row = current->pt.y;
-			ccol = 2 * col + !(row & 1), crow = 2 * row;
 			if (dp->vertical) {
 				dp->shape.hexagon[0].x = dp->xb + ccol * dp->xs;
 				dp->shape.hexagon[0].y = dp->yb + crow * dp->ys;
@@ -420,6 +474,66 @@ draw_state(ModeInfo * mi, int state)
 		XFillRectangles(display, window, gc, rects, ncells);
 		/* Free up rects list and the appropriate part of the cellList */
 		free(rects);
+	} else if (dp->neighbors == 5 || dp->neighbors == 7) {
+		current = dp->cellList[state];
+		int map[4] = {2, 0, 1, 3};
+		while (current) {
+			int col = current->pt.x;
+			int row = current->pt.y;
+			int orient = ((row & 1) * 2 + col) % 4;
+			int offsetX = 0, offsetY = 0;
+
+			switch (orient) {
+#if 0
+			case 0: /* up */
+				offsetX = dp->xs;
+				break;
+			case 1: /* down */
+				offsetY = 4 * dp->ys / 3;
+				break;
+			case 2: /* left */
+				offsetX = -dp->xs / 3;
+				offsetY = 2 * dp->ys / 3;
+				break;
+			case 3: /* right */
+				offsetX = 4 * dp->xs / 3;
+				offsetY = 2 * dp->ys / 3;
+				break;
+#else
+			case 0: /* up */
+				offsetX = dp->xs;
+				offsetY = -1;
+				break;
+			case 1: /* down */
+				offsetY = dp->ys;
+				break;
+			case 2: /* left */
+				offsetX = -2;
+				offsetY = dp->ys / 2;
+				break;
+			case 3: /* right */
+				offsetX = dp->xs;
+				offsetY = dp->ys / 2;
+				break;
+#endif
+			default:
+				(void) printf("wrong orient %d\n", orient);
+			}
+			orient = map[orient];
+			dp->shape.pentagon[orient][0].x =  dp->xb +
+				col * dp->xs + offsetX;
+			dp->shape.pentagon[orient][0].y = dp->yb +
+				row * dp->ys + offsetY;
+			if (dp->xs <= 2 || dp->ys <= 2)
+				XDrawPoint(display, window, gc,
+					dp->shape.pentagon[orient][0].x,
+					dp->shape.pentagon[orient][0].y);
+			else
+				XFillPolygon(display, window, gc,
+					dp->shape.pentagon[orient], 5,
+					Convex, CoordModePrevious);
+			current = current->next;
+		}
 	} else {		/* TRI */
 		XGCValues values;
 		GC borderGC = XCreateGC(display, window, 0, &values);
@@ -493,6 +607,365 @@ RandomSoup(ModeInfo * mi)
 	}
 }
 
+static int
+positionOfNeighbor(demonstruct * dp, int n, int col, int row)
+{
+	int dir = n * (360 / dp->neighbors);
+
+	if (dp->polygon == 4 || dp->polygon == 6) {
+		switch (dir) {
+		case 0:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			break;
+		case 45:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 60:
+			if (!(row & 1))
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 90:
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 120:
+			if (row & 1)
+				col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 135:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 180:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			break;
+		case 225:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 240:
+			if (row & 1)
+				col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 270:
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 300:
+			if (!(row & 1))
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 315:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (dp->polygon == 3) {
+		if ((col + row) & 1) {	/* right */
+			switch (dir) {
+			case 0:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				break;
+			case 30:
+			case 40:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 60:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 80:
+			case 90:
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 120:
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 150:
+			case 160:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 180:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				break;
+			case 200:
+			case 210:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 240:
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 270:
+			case 280:
+				if (!row)
+					row = dp->nrows - 2;
+				else if (!(row - 1))
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 300:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				if (!row)
+					row = dp->nrows - 2;
+				else if (!(row - 1))
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 320:
+			case 330:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+		} else {	/* left */
+			switch (dir) {
+			case 0:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				break;
+			case 30:
+			case 40:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 60:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				if (!row)
+					row = dp->nrows - 2;
+				else if (row == 1)
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 80:
+			case 90:
+				if (!row)
+					row = dp->nrows - 2;
+				else if (row == 1)
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 120:
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 150:
+			case 160:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 180:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				break;
+			case 200:
+			case 210:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 240:
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 270:
+			case 280:
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 300:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 320:
+			case 330:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+		}
+	} else {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= dp->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= dp->ncols) ? 1 : col + 3;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? dp->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? dp->ncols - 2 : col - 3;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? dp->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= dp->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
+	}
+	return (row * dp->ncols + col);
+}
+
 void
 init_demon(ModeInfo * mi)
 {
@@ -569,6 +1042,7 @@ init_demon(ModeInfo * mi)
 	if (dp->neighbors == 6) {
 		int nccols, ncrows, i;
 
+		dp->polygon = 6;
 		if (!dp->vertical) {
 			dp->height = MI_WIDTH(mi);
 			dp->width = MI_HEIGHT(mi);
@@ -612,6 +1086,7 @@ init_demon(ModeInfo * mi)
 			}
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
+		dp->polygon = 4;
 		if (size < -MINSIZE)
 			dp->ys = NRAND(MIN(-size, MAX(MINSIZE, MIN(dp->width, dp->height) /
 				      MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
@@ -628,9 +1103,51 @@ init_demon(ModeInfo * mi)
 		dp->nrows = MAX(dp->height / dp->ys, 2);
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
+	} else if (dp->neighbors == 5 || dp->neighbors == 7) {
+				int orient, sides;
+		dp->polygon = 5;
+		if (dp->width < 2)
+			dp->width = 2;
+		if (dp->height < 2)
+			dp->height = 2;
+		if (size < -MINSIZE)
+			dp->xs = NRAND(MIN(-size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+				MINGRIDSIZE)) - MINSIZE + 1) + MINSIZE;
+		else if (size < MINSIZE) {
+			if (!size) {
+				int min = MIN(dp->width, dp->height) / (4 * MINGRIDSIZE);
+				int max = MIN(dp->width, dp->height) / (MINGRIDSIZE);
+
+				dp->xs = MAX(MINSIZE, min + NRAND(max - min + 1));
+			} else
+				dp->xs = MINSIZE;
+		} else
+			dp->xs = MIN(size, MAX(MINSIZE, MIN(dp->width, dp->height) /
+				MINGRIDSIZE));
+		dp->ys = dp->xs * 2;
+		dp->ncols = (MAX(dp->width / dp->xs, 8) / 4) * 4;
+		dp->nrows = (MAX(dp->height / dp->ys, 8)) / 2 * 2;
+		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
+		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
+		for (orient = 0; orient < 4; orient++) {
+			for (sides = 0; sides < 5; sides++) {
+#if 0
+				dp->shape.pentagon[orient][sides].x =
+					2 * dp->xs * pentagonUnit[orient][sides].x / 3;
+				dp->shape.pentagon[orient][sides].y =
+					dp->ys * pentagonUnit[orient][sides].y / 3;
+#else
+				dp->shape.pentagon[orient][sides].x =
+					dp->xs * pentagonUnit[orient][sides].x / 2;
+				dp->shape.pentagon[orient][sides].y =
+					dp->ys * pentagonUnit[orient][sides].y / 4;
+#endif
+			}
+		}
 	} else {		/* TRI */
 		int orient, i;
 
+		dp->polygon = 3;
 		if (!dp->vertical) {
 			dp->height = MI_WIDTH(mi);
 			dp->width = MI_HEIGHT(mi);
@@ -692,7 +1209,7 @@ init_demon(ModeInfo * mi)
 void
 draw_demon(ModeInfo * mi)
 {
-	int         i, j, k, l, mj = 0, ml;
+	int         i, j, k, l, n, mj = 0;
 	demonstruct *dp;
 
 	if (demons == NULL)
@@ -706,300 +1223,15 @@ draw_demon(ModeInfo * mi)
 		(void) memcpy((char *) dp->newcell, (char *) dp->oldcell,
 			      dp->ncols * dp->nrows * sizeof (unsigned char));
 
-		if (dp->neighbors == 6) {
-			for (j = 0; j < dp->nrows; j++) {
-				for (i = 0; i < dp->ncols; i++) {
-					/* NE */
-					if (!(j & 1))
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-					else
-						k = i;
-					l = (!j) ? dp->nrows - 1 : j - 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* E */
-					k = (i + 1 == dp->ncols) ? 0 : i + 1;
-					ml = mj;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* SE */
-					if (!(j & 1))
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-					else
-						k = i;
-					l = (j + 1 == dp->nrows) ? 0 : j + 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* SW */
-					if (j & 1)
-						k = (!i) ? dp->ncols - 1 : i - 1;
-					else
-						k = i;
-					l = (j + 1 == dp->nrows) ? 0 : j + 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* W */
-					k = (!i) ? dp->ncols - 1 : i - 1;
-					ml = mj;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* NW */
-					if (j & 1)
-						k = (!i) ? dp->ncols - 1 : i - 1;
-					else
-						k = i;
-					l = (!j) ? dp->nrows - 1 : j - 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-				}
-				mj += dp->ncols;
-			}
-		} else if (dp->neighbors == 4 || dp->neighbors == 8) {
-			for (j = 0; j < dp->nrows; j++) {
-				for (i = 0; i < dp->ncols; i++) {
-					/* N */
-					k = i;
-					l = (!j) ? dp->nrows - 1 : j - 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* E */
-					k = (i + 1 == dp->ncols) ? 0 : i + 1;
-					ml = mj;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* S */
-					k = i;
-					l = (j + 1 == dp->nrows) ? 0 : j + 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* W */
-					k = (!i) ? dp->ncols - 1 : i - 1;
-					l = j;
-					ml = mj;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-				}
-				mj += dp->ncols;
-			}
-			if (dp->neighbors == 8) {
-				mj = 0;
-				for (j = 0; j < dp->nrows; j++) {
-					for (i = 0; i < dp->ncols; i++) {
-						/* NE */
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-						l = (!j) ? dp->nrows - 1 : j - 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* SE */
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-						l = (j + 1 == dp->nrows) ? 0 : j + 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* SW */
-						k = (!i) ? dp->ncols - 1 : i - 1;
-						l = (j + 1 == dp->nrows) ? 0 : j + 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* NW */
-						k = (!i) ? dp->ncols - 1 : i - 1;
-						l = (!j) ? dp->nrows - 1 : j - 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-					}
-					mj += dp->ncols;
-				}
-			}
-		} else if (dp->neighbors == 3 || dp->neighbors == 9 ||
-			   dp->neighbors == 12) {
-			for (j = 0; j < dp->nrows; j++) {
-				for (i = 0; i < dp->ncols; i++) {
-					if ((i + j) % 2) {	/* right */
-						/* W */
-						k = (!i) ? dp->ncols - 1 : i - 1;
-						ml = mj;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-					} else {	/* left */
-						/* E */
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-						ml = mj;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-					}
-					/* N */
-					k = i;
-					l = (!j) ? dp->nrows - 1 : j - 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-					/* S */
-					k = i;
-					l = (j + 1 == dp->nrows) ? 0 : j + 1;
-					ml = l * dp->ncols;
-					if (dp->oldcell[k + ml] ==
-					    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-						dp->newcell[i + mj] = dp->oldcell[k + ml];
-				}
-				mj += dp->ncols;
-			}
-			if (dp->neighbors == 9 || dp->neighbors == 12) {
-				mj = 0;
-				for (j = 0; j < dp->nrows; j++) {
-					for (i = 0; i < dp->ncols; i++) {
-						/* NN */
-						k = i;
-						if (!j)
-							l = dp->nrows - 2;
-						else if (!(j - 1))
-							l = dp->nrows - 1;
-						else
-							l = j - 2;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* SS */
-						k = i;
-						if (j + 1 == dp->nrows)
-							l = 1;
-						else if (j + 2 == dp->nrows)
-							l = 0;
-						else
-							l = j + 2;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* NW */
-						k = (!i) ? dp->ncols - 1 : i - 1;
-						l = (!j) ? dp->nrows - 1 : j - 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* NE */
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-						l = (!j) ? dp->nrows - 1 : j - 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* SW */
-						k = (!i) ? dp->ncols - 1 : i - 1;
-						l = (j + 1 == dp->nrows) ? 0 : j + 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-						/* SE */
-						k = (i + 1 == dp->ncols) ? 0 : i + 1;
-						l = (j + 1 == dp->nrows) ? 0 : j + 1;
-						ml = l * dp->ncols;
-						if (dp->oldcell[k + ml] ==
-						    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-							dp->newcell[i + mj] = dp->oldcell[k + ml];
-					}
-					mj += dp->ncols;
-				}
-				if (dp->neighbors == 12) {
-					mj = 0;
-					for (j = 0; j < dp->nrows; j++) {
-						for (i = 0; i < dp->ncols; i++) {
-							if ((i + j) % 2) {	/* right */
-								/* NNW */
-								k = (!i) ? dp->ncols - 1 : i - 1;
-								if (!j)
-									l = dp->nrows - 2;
-								else if (!(j - 1))
-									l = dp->nrows - 1;
-								else
-									l = j - 2;
-								ml = l * dp->ncols;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-								/* SSW */
-								k = (!i) ? dp->ncols - 1 : i - 1;
-								if (j + 1 == dp->nrows)
-									l = 1;
-								else if (j + 2 == dp->nrows)
-									l = 0;
-								else
-									l = j + 2;
-								ml = l * dp->ncols;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-								/* EE */
-								k = (i + 1 == dp->ncols) ? 0 : i + 1;
-								l = j;
-								ml = mj;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-							} else {	/* left */
-								/* NNE */
-								k = (i + 1 == dp->ncols) ? 0 : i + 1;
-								if (!j)
-									l = dp->nrows - 2;
-								else if (!(j - 1))
-									l = dp->nrows - 1;
-								else
-									l = j - 2;
-								ml = l * dp->ncols;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-								/* SSE */
-								k = (i + 1 == dp->ncols) ? 0 : i + 1;
-								if (j + 1 == dp->nrows)
-									l = 1;
-								else if (j + 2 == dp->nrows)
-									l = 0;
-								else
-									l = j + 2;
-								ml = l * dp->ncols;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-								/* WW */
-								k = (!i) ? dp->ncols - 1 : i - 1;
-								l = j;
-								ml = mj;
-								if (dp->oldcell[k + ml] ==
-								    (int) (dp->oldcell[i + mj] + 1) % dp->states)
-									dp->newcell[i + mj] = dp->oldcell[k + ml];
-							}
-						}
-						mj += dp->ncols;
-					}
+		for (j = 0; j < dp->nrows; j++) {
+			mj =  dp->ncols * j;
+			for (i = 0; i < dp->ncols; i++) {
+				for (n = 0; n < dp->neighbors; n++) {
+					l = positionOfNeighbor(dp, n, i, j);
+					k = i + mj;
+					if (dp->oldcell[l] ==
+						    (int) (dp->oldcell[k] + 1) % dp->states)
+						dp->newcell[k] = dp->oldcell[l];
 				}
 			}
 		}
@@ -1029,7 +1261,7 @@ draw_demon(ModeInfo * mi)
 	if (dp->redrawing) {
 		for (i = 0; i < REDRAWSTEP; i++) {
 			if (dp->oldcell[dp->redrawpos]) {
-				drawcell(mi, dp->redrawpos % dp->ncols, dp->redrawpos / dp->ncols,
+				drawCell(mi, dp->redrawpos % dp->ncols, dp->redrawpos / dp->ncols,
 					 dp->oldcell[dp->redrawpos]);
 			}
 			if (++(dp->redrawpos) >= dp->ncols * dp->nrows) {
