@@ -187,7 +187,7 @@ typedef struct {
 	int         ncols, nrows;
 	int         npositions;
 	int         width, height;
-	int         neighbors;
+	int         neighbors, polygon;
 	union {
 		XPoint      hexagon[7];
 		XPoint      triangle[2][4];
@@ -465,227 +465,367 @@ alloc_dilemma(dilemmastruct *dp)
 	}
 }
 
+
 static int
-neighbor_position(dilemmastruct * dp, int col, int row, int dir)
+positionOfNeighbor(dilemmastruct * dp, int n, int col, int row)
 {
-	if (dp->neighbors == 6) {
+	int dir = n * (360 / dp->neighbors);
+
+	if (dp->polygon == 4 || dp->polygon == 6) {
 		switch (dir) {
-			case 0:
+		case 0:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			break;
+		case 45:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 60:
+			if (!(row & 1))
 				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 90:
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 120:
+			if (row & 1)
+				col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 135:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (!row) ? dp->nrows - 1 : row - 1;
+			break;
+		case 180:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			break;
+		case 225:
+			col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 240:
+			if (row & 1)
+				col = (!col) ? dp->ncols - 1 : col - 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 270:
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 300:
+			if (!(row & 1))
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		case 315:
+			col = (col + 1 == dp->ncols) ? 0 : col + 1;
+			row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (dp->polygon == 3) {
+		if ((col + row) & 1) {	/* right */
+			switch (dir) {
+			case 0:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				break;
+			case 30:
+			case 40:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
 				break;
 			case 60:
-				if (!(row & 1))
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-				row = (!row) ? dp->nrows - 1 : row - 1;
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 80:
+			case 90:
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
 				break;
 			case 120:
-				if (row & 1)
-					col = (!col) ? dp->ncols - 1 : col - 1;
-				row = (!row) ? dp->nrows - 1 : row - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 150:
+			case 160:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
 				break;
 			case 180:
-				col = (!col) ? dp->ncols - 1 : col - 1;
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				break;
+			case 200:
+			case 210:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
 				break;
 			case 240:
-				if (row & 1)
-					col = (!col) ? dp->ncols - 1 : col - 1;
-				row = (row + 1 == dp->nrows) ? 0 : row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-				row = (row + 1 == dp->nrows) ? 0 : row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
-		switch (dir) {
-			case 0:
-				col = (col + 1 == dp->ncols) ? 0 : col + 1;
-				break;
-			case 45:
-				col = (col + 1 == dp->ncols) ? 0 : col + 1;
 				row = (!row) ? dp->nrows - 1 : row - 1;
 				break;
-			case 90:
-				row = (!row) ? dp->nrows - 1 : row - 1;
-				break;
-			case 135:
-				col = (!col) ? dp->ncols - 1 : col - 1;
-				row = (!row) ? dp->nrows - 1 : row - 1;
-				break;
-			case 180:
-				col = (!col) ? dp->ncols - 1 : col - 1;
-				break;
-			case 225:
-				col = (!col) ? dp->ncols - 1 : col - 1;
-				row = (row + 1 == dp->nrows) ? 0 : row + 1;
-				break;
 			case 270:
-				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			case 280:
+				if (!row)
+					row = dp->nrows - 2;
+				else if (!(row - 1))
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
 				break;
-			case 315:
-				col = (col + 1 == dp->ncols) ? 0 : col + 1;
-				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+			case 300:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				if (!row)
+					row = dp->nrows - 2;
+				else if (!(row - 1))
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 320:
+			case 330:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
 				break;
 			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else {		/* TRI */
-		if ((col + row) % 2) {	/* right */
-			switch (dir) {
-				case 0:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					break;
-				case 30:
-				case 40:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				case 60:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					if (row + 1 == dp->nrows)
-						row = 1;
-					else if (row + 2 == dp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 80:
-				case 90:
-					if (row + 1 == dp->nrows)
-						row = 1;
-					else if (row + 2 == dp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 120:
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				case 150:
-				case 160:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				case 180:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					break;
-				case 200:
-				case 210:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				case 240:
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				case 270:
-				case 280:
-					if (!row)
-						row = dp->nrows - 2;
-					else if (!(row - 1))
-						row = dp->nrows - 1;
-					else
-						row = row - 2;
-
-					break;
-				case 300:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					if (!row)
-						row = dp->nrows - 2;
-					else if (!(row - 1))
-						row = dp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 320:
-				case 330:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		} else {	/* left */
 			switch (dir) {
-				case 0:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					break;
-				case 30:
-				case 40:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				case 60:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					if (!row)
-						row = dp->nrows - 2;
-					else if (row == 1)
-						row = dp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 80:
-				case 90:
-					if (!row)
-						row = dp->nrows - 2;
-					else if (row == 1)
-						row = dp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 120:
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				case 150:
-				case 160:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					row = (!row) ? dp->nrows - 1 : row - 1;
-					break;
-				case 180:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					break;
-				case 200:
-				case 210:
-					col = (!col) ? dp->ncols - 1 : col - 1;
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				case 240:
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				case 270:
-				case 280:
-					if (row + 1 == dp->nrows)
-						row = 1;
-					else if (row + 2 == dp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 300:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					if (row + 1 == dp->nrows)
-						row = 1;
-					else if (row + 2 == dp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 320:
-				case 330:
-					col = (col + 1 == dp->ncols) ? 0 : col + 1;
-					row = (row + 1 == dp->nrows) ? 0 : row + 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			case 0:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				break;
+			case 30:
+			case 40:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 60:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				if (!row)
+					row = dp->nrows - 2;
+				else if (row == 1)
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 80:
+			case 90:
+				if (!row)
+					row = dp->nrows - 2;
+				else if (row == 1)
+					row = dp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 120:
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 150:
+			case 160:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (!row) ? dp->nrows - 1 : row - 1;
+				break;
+			case 180:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				break;
+			case 200:
+			case 210:
+				col = (!col) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 240:
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 270:
+			case 280:
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 300:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				if (row + 1 == dp->nrows)
+					row = 1;
+				else if (row + 2 == dp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 320:
+			case 330:
+				col = (col + 1 == dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		}
+#if 0
+	} else {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= dp->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= dp->ncols) ? 1 : col + 3;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? dp->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? dp->ncols - 2 : col - 3;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? dp->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? dp->ncols - 1 : col - 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row + 1 == dp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= dp->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= dp->ncols) ? 0 : col + 1;
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? dp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
+#endif
 	}
-
-	return row * dp->ncols + col;
+	return (row * dp->ncols + col);
 }
-
 static void
 draw_state(ModeInfo * mi, int state)
 {
@@ -769,6 +909,7 @@ init_dilemma(ModeInfo * mi)
 	if (dp->neighbors == 6) {
 		int nccols, ncrows, sides;
 
+		dp->polygon = 6;
 		if (!dp->vertical) {
 			dp->height = MI_WIDTH(mi);
 			dp->width = MI_HEIGHT(mi);
@@ -813,6 +954,7 @@ init_dilemma(ModeInfo * mi)
 			}
 		}
 	} else if (dp->neighbors == 4 || dp->neighbors == 8) {
+		dp->polygon = 4;
 		if (dp->width < 2)
 			dp->width = 2;
 		if (dp->height < 2)
@@ -846,8 +988,9 @@ init_dilemma(ModeInfo * mi)
 		dp->xb = (dp->width - dp->xs * dp->ncols) / 2;
 		dp->yb = (dp->height - dp->ys * dp->nrows) / 2;
 	} else {		/* TRI */
-		int orient, sides;
+		int orient;
 
+		dp->polygon = 3;
 		if (!dp->vertical) {
 			dp->height = MI_WIDTH(mi);
 			dp->width = MI_HEIGHT(mi);
@@ -980,8 +1123,8 @@ draw_dilemma(ModeInfo * mi)
 				for (n = 0; n < dp->neighbors; n++)
 					dp->payoff[colrow] +=
 						dp->pm[(int) dp->s[colrow]][(int)
-						  dp->s[neighbor_position(dp,
-					col, row, n * 360 / dp->neighbors)]];
+						  dp->s[positionOfNeighbor(dp,
+					n, col, row)]];
 
 			}
 		}
@@ -994,7 +1137,7 @@ draw_dilemma(ModeInfo * mi)
 				hp = dp->payoff[colrow];
 				dp->sn[colrow] = dp->s[colrow];
 				for (n = 0; n < dp->neighbors; n++) {
-					position = neighbor_position(dp, col, row, n * 360 / dp->neighbors);
+					position = positionOfNeighbor(dp, n, col, row);
 					if (ROUND_FLOAT(dp->payoff[position], 0.001) >
 					    ROUND_FLOAT(hp, 0.001)) {
 						hp = dp->payoff[position];
