@@ -213,11 +213,15 @@ ModStruct   glplanet_description =
 
 /* structure for holding the planet data */
 typedef struct {
+#ifdef WIN32
+  HGLRC glx_context;
+#else
+  GLXContext *glx_context;
+#endif
+  Window window;
   GLuint platelist;
   GLuint starlist;
   int screen_width, screen_height;
-  GLXContext *glx_context;
-  Window window;
 
   XColor fg, bg;
 
@@ -718,9 +722,11 @@ draw_glplanet(ModeInfo * mi)
 
   glDrawBuffer(GL_BACK);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
-
+#ifdef WIN32
+  wglMakeCurrent(hdc, gp->glx_context);
+#else
   glXMakeCurrent(display, window, *(gp->glx_context));
-
+#endif
 
   if (do_stars) {
 	/* protect our modelview matrix and attributes */
@@ -774,7 +780,11 @@ release_glplanet(ModeInfo * mi)
 
 	  if (gp->glx_context) {
 		/* Display lists MUST be freed while their glXContext is current. */
+#ifdef WIN32
+		wglMakeCurrent(hdc, gp->glx_context);
+#else
 		glXMakeCurrent(MI_DISPLAY(mi), gp->window, *(gp->glx_context));
+#endif
 
 		if (glIsList(gp->platelist))
 		  glDeleteLists(gp->platelist, 1);
