@@ -1186,8 +1186,11 @@ void
 	}
     }
     glDrawBuffer(GL_BACK);
+#ifdef WIN32
+    wglMakeCurrent(hdc, tp->glx_context);
+#else
     glXMakeCurrent(display, window, *(tp->glx_context));
-
+#endif
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
     Draw(tp, display, window);
@@ -1246,7 +1249,11 @@ void
     if (!tp->glx_context)
 	return;
     glDrawBuffer(GL_BACK);
+#ifdef WIN32
+    wglMakeCurrent(hdc, tp->glx_context);
+#else
     glXMakeCurrent(MI_DISPLAY(mi), MI_WINDOW(mi), *(tp->glx_context));
+#endif
 }
 
 #endif				/* MODE_text3d */
