@@ -447,9 +447,6 @@ figure_points(ModeInfo * mi)
 	 * First, figure out the points.
 	 */
 	hp->num_points = figure_num_points(hp->num_d);
-#ifdef DEBUG
-	assert(hp->num_points <= hp->max_points);
-#endif
 	allocarray(hp->points, vector, hp->num_points * hp->num_mat);
 	allocarray(hp->xpoints[0], XPoint, hp->num_points);
 	allocarray(hp->xpoints[1], XPoint, hp->num_points);
@@ -484,9 +481,6 @@ figure_points(ModeInfo * mi)
 		pix = NRAND(MI_NPIXELS(mi));
 
 	hp->num_lines = figure_num_lines(hp->num_d);
-#ifdef DEBUG
-	assert(hp->num_lines <= hp->max_lines);
-#endif
 	allocarray(hp->lines, line_segment, hp->num_lines);
 	for (n = i = 0; i < hp->num_points; i++) {
 		for (j = i + 1; j < hp->num_points; j++) {
@@ -516,9 +510,6 @@ figure_points(ModeInfo * mi)
 	 * Now determine the planes of rotation.
 	 */
 	hp->num_planes = figure_num_planes(hp->num_d);
-#ifdef DEBUG
-	assert(hp->num_planes <= max_planes);
-#endif
 	hp->show_planes = show_planes;
 
 	if (hp->show_planes) {
@@ -1116,7 +1107,7 @@ translate_point(hyperstruct * hp, matrix * Tall, vector * real, XPoint * screen_
 {
 	vector     *image;
 
-    if ((image = (vector *) malloc(hp->num_mat * sizeof (vector))) == NULL)
+	if ((image = (vector *) malloc(hp->num_mat * sizeof (vector))) == NULL)
 		return False;
 	MatVecMult(Tall, real, image, hp->num_mat);
 	screen_image->x = (short) image[0];
