@@ -921,7 +921,7 @@ static unsigned char hex_self_reproducing_loop[HEX_ADAM_LOOPY][HEX_ADAM_LOOPX] =
 };
 
 static void
-position_of_neighbor(loopstruct * lp, int dir, int *pcol, int *prow)
+positionOfNeighbor(loopstruct * lp, int dir, int *pcol, int *prow)
 {
 	int         col = *pcol, row = *prow;
 
@@ -1826,7 +1826,7 @@ init_adam(ModeInfo * mi)
 				break;
 		}
 		for (j = 0; j < SQ_ADAM_LOOPY(evolve); j++)
-			for (i = 0; i < SQ_ADAM_LOOPX(evolve); i++)
+			for (i = 0; i < SQ_ADAM_LOOPX(evolve); i++) {
 			  lp->newcells[lp->bncols * (start.y + dirx.y * i + diry.y * j) +
 			    start.x + dirx.x * i + diry.x * j] =
 			    (evolve) ? ((lp->clockwise) ?
@@ -1836,12 +1836,13 @@ init_adam(ModeInfo * mi)
 			      self_reproducing_loop[j][ADAM_LOOPX - i - 1] :
 			      self_reproducing_loop[j][i]);
 #ifdef DEBUG
-		/* Draw right away */
-		drawcell(mi, start.x + dirx.x * i + diry.x * j - lp->bx,
-		 start.y + dirx.y * i + diry.y * j - lp->by,
-		 (evolve) ? ((lp->clockwise) ? evo_self_reproducing_loop[j][EVO_ADAM_LOOPX - i - 1] : evo_self_reproducing_loop[j][i]) :
-		  ((lp->clockwise) ? self_reproducing_loop[j][ADAM_LOOPX - i - 1] : self_reproducing_loop[j][i]));
+			  /* Draw right away */
+			  drawcell(mi, start.x + dirx.x * i + diry.x * j - lp->bx,
+			   start.y + dirx.y * i + diry.y * j - lp->by,
+			   (evolve) ? ((lp->clockwise) ? evo_self_reproducing_loop[j][EVO_ADAM_LOOPX - i - 1] : evo_self_reproducing_loop[j][i]) :
+			   ((lp->clockwise) ? self_reproducing_loop[j][ADAM_LOOPX - i - 1] : self_reproducing_loop[j][i]));
 #endif
+			}
 	}
 }
 
@@ -1868,7 +1869,7 @@ do_gen(loopstruct * lp)
 			for (k = 0; k < local_neighbors; k++) {
 				int         newi = i, newj = j;
 
-				position_of_neighbor(lp,
+				positionOfNeighbor(lp,
 					k * ANGLES / local_neighbors,
 					&newi, &newj);
 				n[k] = 0;
