@@ -421,9 +421,10 @@ static int check_all_regions_multiple_of(polyominoesstruct *sp, int n) {
     good = count%n == 0;
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (ARRAY(x,y) == -2)
-      ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y) == -2)
+        ARRAY(x,y) = -1;
 
   return good;
 }
@@ -439,9 +440,10 @@ static int check_all_regions_positive_combination_of(polyominoesstruct *sp, int
       good = count%n == 0;
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (ARRAY(x,y) == -2)
-      ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y) == -2)
+        ARRAY(x,y) = -1;
 
   return good;
 }
@@ -507,9 +509,9 @@ score_point(polyominoesstruct *sp, int x, int y, int min_score_so_far) {
   attach_point.x = x;
   attach_point.y = y;
   for (poly_no=first_poly_no(sp);poly_no<sp->nr_polyominoes;next_poly_no(sp,&poly_no))
-  if (!sp->polyomino[poly_no].attached) {
+   if (!sp->polyomino[poly_no].attached) {
     for (point_no=0;point_no<sp->polyomino[poly_no].len;point_no++)
-    for (transform_index=0;transform_index<sp->polyomino[poly_no].transform_len;transform_index++) {
+     for (transform_index=0;transform_index<sp->polyomino[poly_no].transform_len;transform_index++) {
       attachable = 1;
       for (i=0;i<sp->polyomino[poly_no].len;i++) {
         transform(sp->polyomino[poly_no].point[i],
@@ -528,8 +530,8 @@ score_point(polyominoesstruct *sp, int x, int y, int min_score_so_far) {
         if (score>=min_score_so_far)
           return score;
       }
-    }
-  }
+     }
+   }
 
   return score;
 }
@@ -557,8 +559,9 @@ static void find_blank(polyominoesstruct *sp, point_type *point) {
     }
   }
 
-  for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (ARRAY(x,y)<0) ARRAY(x,y) = -1;
+  for (x=0;x<sp->width;x++)
+    for (y=0;y<sp->height;y++)
+      if (ARRAY(x,y)<0) ARRAY(x,y) = -1;
 }
 
 /* Detaches the most recently attached polyomino. */
@@ -693,14 +696,15 @@ draw_without_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
 
   for (poly_no=-1;poly_no<sp->nr_polyominoes;poly_no++) {
     nr_rectangles = 0;
-    for (x=0;x<sp->width;x++) for (y=0;y<sp->height;y++)
-    if (CHANGED_ARRAY(x,y) && ARRAY(x,y) == poly_no) {
-      sp->rectangles[nr_rectangles].x = sp->x_margin + sp->box*x;
-      sp->rectangles[nr_rectangles].y = sp->y_margin + sp->box*y;
-      sp->rectangles[nr_rectangles].width = sp->box;
-      sp->rectangles[nr_rectangles].height = sp->box;
-      nr_rectangles++;
-    }
+    for (x=0;x<sp->width;x++)
+      for (y=0;y<sp->height;y++)
+        if (CHANGED_ARRAY(x,y) && ARRAY(x,y) == poly_no) {
+          sp->rectangles[nr_rectangles].x = sp->x_margin + sp->box*x;
+          sp->rectangles[nr_rectangles].y = sp->y_margin + sp->box*y;
+          sp->rectangles[nr_rectangles].width = sp->box;
+          sp->rectangles[nr_rectangles].height = sp->box;
+          nr_rectangles++;
+        }
     if (poly_no == -1)
       XSetForeground(display, gc, MI_BLACK_PIXEL(mi));
     else
@@ -817,29 +821,37 @@ static void create_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
       }
 
       if (IS_LEFT(n))
-        for (y=0;y<sp->box;y++) for (x=G;x<G+T;x++)
-          SETBIT(n,x,y)
+        for (y=0;y<sp->box;y++)
+          for (x=G;x<G+T;x++)
+            SETBIT(n,x,y)
       if (IS_RIGHT(n))
-        for (y=0;y<sp->box;y++) for (x=G;x<G+T;x++)
-          SETBIT(n,sp->box-1-x,y)
+        for (y=0;y<sp->box;y++)
+          for (x=G;x<G+T;x++)
+            SETBIT(n,sp->box-1-x,y)
       if (IS_UP(n))
-        for (x=0;x<sp->box;x++) for (y=G;y<G+T;y++)
-          SETBIT(n,x,y)
+        for (x=0;x<sp->box;x++)
+          for (y=G;y<G+T;y++)
+            SETBIT(n,x,y)
       if (IS_DOWN(n))
-        for (x=0;x<sp->box;x++) for (y=G;y<G+T;y++)
-          SETBIT(n,x,sp->box-1-y)
+        for (x=0;x<sp->box;x++)
+          for (y=G;y<G+T;y++)
+            SETBIT(n,x,sp->box-1-y)
       if (IS_LEFT(n))
-        for (y=0;y<sp->box;y++) for (x=0;x<G;x++)
-          RESBIT(n,x,y)
+        for (y=0;y<sp->box;y++)
+          for (x=0;x<G;x++)
+            RESBIT(n,x,y)
       if (IS_RIGHT(n))
-        for (y=0;y<sp->box;y++) for (x=0;x<G;x++)
-          RESBIT(n,sp->box-1-x,y)
+        for (y=0;y<sp->box;y++)
+          for (x=0;x<G;x++)
+            RESBIT(n,sp->box-1-x,y)
       if (IS_UP(n))
-        for (x=0;x<sp->box;x++) for (y=0;y<G;y++)
-          RESBIT(n,x,y)
+        for (x=0;x<sp->box;x++)
+          for (y=0;y<G;y++)
+            RESBIT(n,x,y)
       if (IS_DOWN(n))
-        for (x=0;x<sp->box;x++) for (y=0;y<G;y++)
-          RESBIT(n,x,sp->box-1-y)
+        for (x=0;x<sp->box;x++)
+          for (y=0;y<G;y++)
+            RESBIT(n,x,sp->box-1-y)
 
       if (IS_LEFT(n) && IS_UP(n))
         for (x=G;x<=G+R;x++)
@@ -875,69 +887,89 @@ static void create_bitmaps(ModeInfo * mi, polyominoesstruct *sp) {
           }
 
       if (!IS_LEFT(n) && !IS_UP(n) && IS_LEFT_UP(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,x,y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,x,y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,x,y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,x,y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,x,y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,x,y)
       }
       if (!IS_LEFT(n) && !IS_DOWN(n) && IS_LEFT_DOWN(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,x,sp->box-1-y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,x,sp->box-1-y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,x,sp->box-1-y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,x,sp->box-1-y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,x,sp->box-1-y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,x,sp->box-1-y)
       }
       if (!IS_RIGHT(n) && !IS_UP(n) && IS_RIGHT_UP(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,sp->box-1-x,y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,sp->box-1-x,y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,sp->box-1-x,y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,sp->box-1-x,y)
+        for (x=G;x<G+T;x++)
+          for(y=0;y<G;y++)
+            SETBIT(n,sp->box-1-x,y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,sp->box-1-x,y)
       }
       if (!IS_RIGHT(n) && !IS_DOWN(n) && IS_RIGHT_DOWN(n)) {
-        for (x=0;x<G;x++) for(y=0;y<G;y++)
-          RESBIT(n,sp->box-1-x,sp->box-1-y)
-        for (x=G;x<G+T;x++) for(y=0;y<G;y++)
-          SETBIT(n,sp->box-1-x,sp->box-1-y)
-        for (x=0;x<G+T;x++) for(y=G;y<G+T;y++)
-          SETBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=0;x<G;x++)
+          for(y=0;y<G;y++)
+            RESBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=G;x<G+T;x++) 
+          for(y=0;y<G;y++)
+            SETBIT(n,sp->box-1-x,sp->box-1-y)
+        for (x=0;x<G+T;x++)
+          for(y=G;y<G+T;y++)
+            SETBIT(n,sp->box-1-x,sp->box-1-y)
       }
 
 #ifdef LARGE_BELLYBUTTON
       if (!sp->use3D) {
         if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
-          for (x=0;x<G+T;x++) for(y=0;y<G+T;y++)
-            SETBIT(n,x,y)
+          for (x=0;x<G+T;x++)
+            for(y=0;y<G+T;y++)
+              SETBIT(n,x,y)
         if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
-          for (x=0;x<G+T;x++) for(y=sp->box-G-T;y<sp->box;y++)
-            SETBIT(n,x,y)
+          for (x=0;x<G+T;x++)
+            for(y=sp->box-G-T;y<sp->box;y++)
+              SETBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
-          for (x=sp->box-G-T;x<sp->box;x++) for(y=0;y<G+T;y++)
-            SETBIT(n,x,y)
+          for (x=sp->box-G-T;x<sp->box;x++)
+            for(y=0;y<G+T;y++)
+              SETBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
-          for (x=sp->box-G-T;x<sp->box;x++) for(y=sp->box-G-T;y<sp->box;y++)
-            SETBIT(n,x,y)
+          for (x=sp->box-G-T;x<sp->box;x++)
+            for(y=sp->box-G-T;y<sp->box;y++)
+              SETBIT(n,x,y)
       } else
 #else
       if (sp->use3D)
 #endif
       {
         if (!IS_LEFT(n) && !IS_UP(n) && !IS_LEFT_UP(n))
-          for (x=0;x<sp->box/2-RR;x++) for(y=0;y<sp->box/2-RR;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=0;x<sp->box/2-RR;x++)
+            for(y=0;y<sp->box/2-RR;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_LEFT(n) && !IS_DOWN(n) && !IS_LEFT_DOWN(n))
-          for (x=0;x<sp->box/2-RR;x++) for(y=sp->box/2+RR;y<sp->box;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=0;x<sp->box/2-RR;x++)
+            for(y=sp->box/2+RR;y<sp->box;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_UP(n) && !IS_RIGHT_UP(n))
-          for (x=sp->box/2+RR;x<sp->box;x++) for(y=0;y<sp->box/2-RR;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=sp->box/2+RR;x<sp->box;x++)
+            for(y=0;y<sp->box/2-RR;y++)
+              THREEQUARTERSBIT(n,x,y)
         if (!IS_RIGHT(n) && !IS_DOWN(n) && !IS_RIGHT_DOWN(n))
-          for (x=sp->box/2+RR;x<sp->box;x++) for(y=sp->box/2+RR;y<sp->box;y++)
-            THREEQUARTERSBIT(n,x,y)
+          for (x=sp->box/2+RR;x<sp->box;x++)
+            for(y=sp->box/2+RR;y<sp->box;y++)
+              THREEQUARTERSBIT(n,x,y)
       }
 
       sp->bitmaps[n] = XCreateImage(MI_DISPLAY(mi), MI_VISUAL(mi), 1, XYBitmap,
