@@ -166,7 +166,7 @@ typedef struct {
 	int         width, height;
 	CellList   *currkind, *babykind, *lastkind[KINDS + 1], *firstkind[KINDS + 1];
 	CellList  **arr;	/* 0=empty or pts to a fish or shark */
-	int         neighbors;
+	int         neighbors, polygon;
 	union {
 		XPoint      hexagon[6];
 		XPoint      triangle[2][3];
@@ -475,223 +475,364 @@ flush_kindlist(watorstruct * wp, int kind)
 }
 
 static int
-neighbor_position(watorstruct * wp, int col, int row, int dir)
+positionOfNeighbor(watorstruct * wp, int n, int col, int row)
 {
-	if (wp->neighbors == 6) {
+	int dir = n * (360 / wp->neighbors);
+
+	if (wp->polygon == 4 || wp->polygon == 6) {
 		switch (dir) {
-			case 0:
+		case 0:
+			col = (col + 1 == wp->ncols) ? 0 : col + 1;
+			break;
+		case 45:
+			col = (col + 1 == wp->ncols) ? 0 : col + 1;
+			row = (!row) ? wp->nrows - 1 : row - 1;
+			break;
+		case 60:
+			if (!(row & 1))
 				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+			row = (!row) ? wp->nrows - 1 : row - 1;
+			break;
+		case 90:
+			row = (!row) ? wp->nrows - 1 : row - 1;
+			break;
+		case 120:
+			if (row & 1)
+				col = (!col) ? wp->ncols - 1 : col - 1;
+			row = (!row) ? wp->nrows - 1 : row - 1;
+			break;
+		case 135:
+			col = (!col) ? wp->ncols - 1 : col - 1;
+			row = (!row) ? wp->nrows - 1 : row - 1;
+			break;
+		case 180:
+			col = (!col) ? wp->ncols - 1 : col - 1;
+			break;
+		case 225:
+			col = (!col) ? wp->ncols - 1 : col - 1;
+			row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			break;
+		case 240:
+			if (row & 1)
+				col = (!col) ? wp->ncols - 1 : col - 1;
+			row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			break;
+		case 270:
+			row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			break;
+		case 300:
+			if (!(row & 1))
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+			row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			break;
+		case 315:
+			col = (col + 1 == wp->ncols) ? 0 : col + 1;
+			row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			break;
+		default:
+			(void) fprintf(stderr, "wrong direction %d\n", dir);
+		}
+	} else if (wp->polygon == 3) {
+		if ((col + row) & 1) {	/* right */
+			switch (dir) {
+			case 0:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				break;
+			case 30:
+			case 40:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
 				break;
 			case 60:
-				if (!(row & 1))
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-				row = (!row) ? wp->nrows - 1 : row - 1;
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				if (row + 1 == wp->nrows)
+					row = 1;
+				else if (row + 2 == wp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 80:
+			case 90:
+				if (row + 1 == wp->nrows)
+					row = 1;
+				else if (row + 2 == wp->nrows)
+					row = 0;
+				else
+					row = row + 2;
 				break;
 			case 120:
-				if (row & 1)
-					col = (!col) ? wp->ncols - 1 : col - 1;
-				row = (!row) ? wp->nrows - 1 : row - 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 150:
+			case 160:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
 				break;
 			case 180:
-				col = (!col) ? wp->ncols - 1 : col - 1;
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				break;
+			case 200:
+			case 210:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				row = (!row) ? wp->nrows - 1 : row - 1;
 				break;
 			case 240:
-				if (row & 1)
-					col = (!col) ? wp->ncols - 1 : col - 1;
-				row = (row + 1 == wp->nrows) ? 0 : row + 1;
-				break;
-			case 300:
-				if (!(row & 1))
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-				row = (row + 1 == wp->nrows) ? 0 : row + 1;
-				break;
-			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else if (wp->neighbors == 4 || wp->neighbors == 8) {
-		switch (dir) {
-			case 0:
-				col = (col + 1 == wp->ncols) ? 0 : col + 1;
-				break;
-			case 45:
-				col = (col + 1 == wp->ncols) ? 0 : col + 1;
 				row = (!row) ? wp->nrows - 1 : row - 1;
 				break;
-			case 90:
-				row = (!row) ? wp->nrows - 1 : row - 1;
-				break;
-			case 135:
-				col = (!col) ? wp->ncols - 1 : col - 1;
-				row = (!row) ? wp->nrows - 1 : row - 1;
-				break;
-			case 180:
-				col = (!col) ? wp->ncols - 1 : col - 1;
-				break;
-			case 225:
-				col = (!col) ? wp->ncols - 1 : col - 1;
-				row = (row + 1 == wp->nrows) ? 0 : row + 1;
-				break;
 			case 270:
-				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			case 280:
+				if (!row)
+					row = wp->nrows - 2;
+				else if (!(row - 1))
+					row = wp->nrows - 1;
+				else
+					row = row - 2;
 				break;
-			case 315:
-				col = (col + 1 == wp->ncols) ? 0 : col + 1;
-				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+			case 300:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				if (!row)
+					row = wp->nrows - 2;
+				else if (!(row - 1))
+					row = wp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 320:
+			case 330:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				row = (!row) ? wp->nrows - 1 : row - 1;
 				break;
 			default:
-				(void) fprintf(stderr, "wrong direction %d\n", dir);
-		}
-	} else {		/* TRI */
-		if ((col + row) % 2) {	/* right */
-			switch (dir) {
-				case 0:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					break;
-				case 30:
-				case 40:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				case 60:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					if (row + 1 == wp->nrows)
-						row = 1;
-					else if (row + 2 == wp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 80:
-				case 90:
-					if (row + 1 == wp->nrows)
-						row = 1;
-					else if (row + 2 == wp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 120:
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				case 150:
-				case 160:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				case 180:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					break;
-				case 200:
-				case 210:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				case 240:
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				case 270:
-				case 280:
-					if (!row)
-						row = wp->nrows - 2;
-					else if (!(row - 1))
-						row = wp->nrows - 1;
-					else
-						row = row - 2;
-
-					break;
-				case 300:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					if (!row)
-						row = wp->nrows - 2;
-					else if (!(row - 1))
-						row = wp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 320:
-				case 330:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		} else {	/* left */
 			switch (dir) {
-				case 0:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					break;
-				case 30:
-				case 40:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				case 60:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					if (!row)
-						row = wp->nrows - 2;
-					else if (row == 1)
-						row = wp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 80:
-				case 90:
-					if (!row)
-						row = wp->nrows - 2;
-					else if (row == 1)
-						row = wp->nrows - 1;
-					else
-						row = row - 2;
-					break;
-				case 120:
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				case 150:
-				case 160:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					row = (!row) ? wp->nrows - 1 : row - 1;
-					break;
-				case 180:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					break;
-				case 200:
-				case 210:
-					col = (!col) ? wp->ncols - 1 : col - 1;
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				case 240:
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				case 270:
-				case 280:
-					if (row + 1 == wp->nrows)
-						row = 1;
-					else if (row + 2 == wp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 300:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					if (row + 1 == wp->nrows)
-						row = 1;
-					else if (row + 2 == wp->nrows)
-						row = 0;
-					else
-						row = row + 2;
-					break;
-				case 320:
-				case 330:
-					col = (col + 1 == wp->ncols) ? 0 : col + 1;
-					row = (row + 1 == wp->nrows) ? 0 : row + 1;
-					break;
-				default:
-					(void) fprintf(stderr, "wrong direction %d\n", dir);
+			case 0:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				break;
+			case 30:
+			case 40:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				row = (!row) ? wp->nrows - 1 : row - 1;
+				break;
+			case 60:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				if (!row)
+					row = wp->nrows - 2;
+				else if (row == 1)
+					row = wp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 80:
+			case 90:
+				if (!row)
+					row = wp->nrows - 2;
+				else if (row == 1)
+					row = wp->nrows - 1;
+				else
+					row = row - 2;
+				break;
+			case 120:
+				row = (!row) ? wp->nrows - 1 : row - 1;
+				break;
+			case 150:
+			case 160:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				row = (!row) ? wp->nrows - 1 : row - 1;
+				break;
+			case 180:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				break;
+			case 200:
+			case 210:
+				col = (!col) ? wp->ncols - 1 : col - 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 240:
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 270:
+			case 280:
+				if (row + 1 == wp->nrows)
+					row = 1;
+				else if (row + 2 == wp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 300:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				if (row + 1 == wp->nrows)
+					row = 1;
+				else if (row + 2 == wp->nrows)
+					row = 0;
+				else
+					row = row + 2;
+				break;
+			case 320:
+			case 330:
+				col = (col + 1 == wp->ncols) ? 0 : col + 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
 			}
 		}
+#if 0
+	} else {
+		int orient = ((row & 1) * 2 + col) % 4;
+		switch (orient) { /* up, down, left, right */
+		case 0:
+			switch (dir) {
+			case 0:
+				col++;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col + 2 >= wp->ncols) ? 0 : col + 2;
+				break;
+			case 102: /* 7 corner */
+				col = (col + 3 >= wp->ncols) ? 1 : col + 3;
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col++;
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 255: /* 7 */
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 1:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 1) ? wp->ncols - 1 : col - 2;
+				break;
+			case 102: /* 7 */
+				col = (col == 1) ? wp->ncols - 2 : col - 3;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col--;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 2:
+			switch (dir) {
+			case 0:
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				col++;
+				break;
+			case 102: /* 7 */
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col == 0) ? wp->ncols - 2 : col - 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				break;
+			case 255: /* 7 */
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		case 3:
+			switch (dir) {
+			case 0:
+				col--;
+				break;
+			case 51: /* 7 */
+			case 72: /* 5 */
+				col = (col == 0) ? wp->ncols - 1 : col - 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 102: /* 7 */
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				row = (row + 1 == wp->nrows) ? 0 : row + 1;
+				break;
+			case 144: /* 5 */
+			case 153: /* 7 */
+				col = (col + 2 >= wp->ncols) ? 1 : col + 2;
+				break;
+			case 204: /* 7 */
+			case 216: /* 5 */
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				break;
+			case 255: /* 7 */
+				col = (col + 1 >= wp->ncols) ? 0 : col + 1;
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			case 288: /* 5 */
+			case 306: /* 7 */
+				row = (row == 0) ? wp->nrows - 1 : row - 1;
+				break;
+			default:
+				(void) fprintf(stderr, "wrong direction %d\n",
+					dir);
+			}
+			break;
+		default:
+			(void) fprintf(stderr, "wrong orient %d\n",
+				orient);
+		}
+#endif
 	}
-	return row * wp->ncols + col;
+	return (row * wp->ncols + col);
 }
 
 static void
@@ -781,6 +922,7 @@ init_wator(ModeInfo * mi)
 	if (wp->neighbors == 6) {
 		int nccols, ncrows, sides;
 
+		wp->polygon = 6;
 		if (!wp->vertical) {
 			wp->height = MI_WIDTH(mi);
 			wp->width = MI_HEIGHT(mi);
@@ -825,6 +967,7 @@ init_wator(ModeInfo * mi)
 			}
 		}
 	} else if (wp->neighbors == 4 || wp->neighbors == 8) {
+		wp->polygon = 4;
 		if (wp->width < 2)
 			wp->width = 2;
 		if (wp->height < 2)
@@ -860,6 +1003,7 @@ init_wator(ModeInfo * mi)
 	} else {		/* TRI */
 		int orient, sides;
 
+		wp->polygon = 3;
 		if (!wp->vertical) {
 			wp->height = MI_WIDTH(mi);
 			wp->width = MI_HEIGHT(mi);
@@ -990,7 +1134,7 @@ draw_wator(ModeInfo * mi)
 			numok = 0;
 			if (wp->kind == SHARK) {	/* Scan for fish */
 				for (i = 0; i < wp->neighbors; i++) {
-					position = neighbor_position(wp, col, row, i * 360 / wp->neighbors);
+					position = positionOfNeighbor(wp, i, col, row);
 					if (wp->arr[position] && wp->arr[position]->info.kind == FISH) {
 						acell[numok].pos = position;
 						acell[numok++].dir = i;
@@ -1052,7 +1196,7 @@ draw_wator(ModeInfo * mi)
 			}
 			if (!numok) {	/* Fish or shark search for a place to go */
 				for (i = 0; i < wp->neighbors; i++) {
-					position = neighbor_position(wp, col, row, i * 360 / wp->neighbors);
+					position = positionOfNeighbor(wp, i, col, row);
 					if (!wp->arr[position]) {	/* Found an empty spot */
 						acell[numok].pos = position;
 						acell[numok++].dir = i;
