@@ -232,14 +232,15 @@ static void view_3d(XSegment *xseg, xclstruct *dp)
   }
 }
 
-static long get_color(Display *dpy, char *color, XColor *final_color)
+static long get_color(ModeInfo *mi, char *color, XColor *final_color)
 {
+  Display *display = MI_DISPLAY(mi);
   XColor cdef;
   Colormap cmap;
-  cmap = DefaultColormap(dpy,DefaultScreen(dpy));
+  cmap = DefaultColormap(display, MI_SCREEN(mi));
 
-  if (!XParseColor(dpy, cmap, color, &cdef) ||
-      !XAllocColor(dpy, cmap, &cdef))
+  if (!XParseColor(display, cmap, color, &cdef) ||
+      !XAllocColor(display, cmap, &cdef))
     {
       (void) fprintf(stderr, "Color \"%s\" wasn't found\n", color);
     }
@@ -383,11 +384,11 @@ void init_xcl(ModeInfo * mi)
         for(i=0;i< dp->planes; i++) {
           switch(i) {
           case 0:
-            dp->planecolor[0] = get_color(display, (char *) "yellow",
+            dp->planecolor[0] = get_color(mi, (char *) "yellow",
 		(XColor *) NULL);
             break;
           case 1:
-            dp->planecolor[1] = get_color(display, (char *) "red",
+            dp->planecolor[1] = get_color(mi, (char *) "red",
 		(XColor *) NULL);
             break;
           default:
@@ -397,7 +398,7 @@ void init_xcl(ModeInfo * mi)
       }
     }
 
-    if(dp->erase_gc == NULL)
+    if(dp->erase_gc == None)
       if (!get_GC(display, MI_WINDOW(mi), &(dp->erase_gc),dp->bg)) {
         free_xcl(display, dp);
         return;
@@ -406,7 +407,7 @@ void init_xcl(ModeInfo * mi)
     dp->lines = countlines();
 
     for(i=0;i< dp->planes; i++) {
-      if(dp->gc[i] == NULL)
+      if(dp->gc[i] == None)
         if (!get_GC(display, MI_WINDOW(mi), &(dp->gc[i]),
                     dp->planecolor[i])) {
           free_xcl(display, dp);
