@@ -106,12 +106,12 @@ HANDLE thread;                         /* thread handle */
 DWORD thread_id;                       /* thread id number */
 
 int      numGCInfo = 0;                /* number of GC Info */
-TGCInfo gcInfo[NUMBER_GC + 1] = { 0 }; /* information about GCs. Allow
+TGCInfo gcInfo[NUMBER_GC + 1] = { {0} }; /* information about GCs. Allow
                                           NUMBER_GC GC's, + 1 as a spill
 										  over for any modes that need
 										  more */
 int      numBMInfo = 0;                /* number of Bitmap Info */
-TBMInfo bmInfo[NUMBER_BITMAP + 1] = { 0 }; /* information about Bitmaps/Pixmaps */
+TBMInfo bmInfo[NUMBER_BITMAP + 1] = { {0} }; /* information about Bitmaps/Pixmaps */
 
 /* -------------------------------------------------------------------- */
 
@@ -130,7 +130,7 @@ void SetByte(LPBYTE lpBMData, BYTE val, int loc, int maxLoc)
 void SetGraphic(LPBYTE lpBMData, XImage *image)
 {
   const int w32bb = 4 * 8;
-  const int xbmbb = 1 * 8;
+  /*const int xbmbb = 1 * 8;*/
   BYTE xbmval;
   BYTE w32val;
   int lineBitCnt = 0;
@@ -547,7 +547,9 @@ HDC GCGetDC(Drawable d, GC gc)
 	}
 
     /* check if we need a pattern brush or solid brush */
-	if (gcInfo[gc].stipple.hasStipple == FALSE)
+	if (gcInfo[gc].stipple.hasStipple == FALSE ||
+		(gcInfo[gc].fillStyle.hasFillStyle == TRUE &&
+		 gcInfo[gc].fillStyle.value == FillSolid))
 	{
 		/* solid brush */
 		brush = SelectObject(lhdc,
@@ -693,7 +695,6 @@ HDC GCGetDC(Drawable d, GC gc)
 int BMInfoCreate(void)
 {
 	int retVal	= 0;
-	HBITMAP hBitmap;
 
 	/* start at 1, not 0 as this will be None */
 	for (retVal=1; retVal<NUMBER_BITMAP; retVal++)
@@ -928,7 +929,7 @@ Status XAllocColor(Display *display, Colormap colormap,
  *    not currently used
  */
 Status XAllocColorCells(Display *display, Colormap colormap,
-					    Bool contig, unsigned longplane_masks_return[],
+					    Bool contig, unsigned long plane_masks_return[],
 						unsigned int nplanes, unsigned long pixels_return[],
 						unsigned int npixels)
 {
@@ -1097,8 +1098,8 @@ int XCopyArea(Display *display, Drawable src, Drawable dest, GC gc,
 			  int dest_x, int dest_y)
 {
 	HBITMAP	hBitmap;
-	HBITMAP	hOldBitmapS;
-	HBITMAP	hOldBitmapD;
+	HBITMAP	hOldBitmapS = NULL;
+	HBITMAP	hOldBitmapD = NULL;
 	HDC		hDestDC;
 	HDC		hSrcDC;
 
@@ -1253,6 +1254,18 @@ GC XCreateGC(Display *display, Drawable drawable,
 	}
 	else
 	{
+		/* check if drawable is a bitmap, and adjust the 1 and 0 colors
+		   (white and black) to their proper win32 values */
+		if (drawable > 0 && drawable < NUMBER_BITMAP)
+			if (bmInfo[drawable].isActive == TRUE && bmInfo[drawable].type == 1)
+			{
+				if (valuemask & GCBackground)
+					if (values->background == 0 || values->background == 1)
+						values->background = 256 + values->background;
+				if (valuemask & GCForeground)
+					if (values->foreground == 0 || values->foreground == 1)
+						values->foreground = 256 + values->foreground;
+			}
 		XChangeGC(display, newGc, valuemask, values);
 	}
 
@@ -1303,14 +1316,16 @@ Pixmap XCreatePixmap(Display *display, Drawable d, unsigned int width,
 		return None;
 
 	/* create the bitmap */
-	if ((hBitmap = CreateCompatibleBitmap(hdc, width, height)) == NULL)
+	if ((hBitmap = 
+		(depth == 1) ? CreateBitmap(width, height, depth, 1, NULL) : 
+					   CreateCompatibleBitmap(hdc, width, height)) == NULL)
 	{
 		bmInfo[pm].isActive = FALSE;
  		return None;
 	}
 
 	bmInfo[pm].bitmap = hBitmap;
-	bmInfo[pm].type = 0;
+	bmInfo[pm].type = depth;
 	bmInfo[pm].width = width;
 	bmInfo[pm].height = height;
 
@@ -1340,7 +1355,7 @@ Pixmap XCreatePixmapFromBitmapData(Display *display, Drawable drawable,
 	HDC		hBitmapDC;
 	HBITMAP	hOldBitmap;
 	Pixmap	pm;
-	RGBQUAD	rgbQuad[2] = { 0 };
+	RGBQUAD	rgbQuad[2] = { {0} };
 
 	/* create the bitmap */
 	if ((pm = XCreateBitmapFromData(display, drawable, data, width, height)) == None)
@@ -1422,8 +1437,8 @@ void XDrawArc(Display *display, Drawable d, GC gc, int x, int y,
 
 	rx = width/2.0;
 	ry = height/2.0;
-	cx = x + width/2;
-	cy = y + height/2;
+	cx = x + width/2 + 2;
+	cy = y + height/2 + 2;
 
 	a1 = (23040 - (angle1 % 23040)) * ang_inc;
 
@@ -2349,6 +2364,60 @@ void XPutImage(Display *display, Drawable d, GC gc, XImage *image, int src_x,
 	}
 }
 
+/*-
+ *  XPutPixel
+ *    Set a pixels color value on an image
+ *
+ *  NOTE: I'm not sure what this function returns. The header files
+ *  under cygwin says it should return int, but some references out
+ *  on the web indicate void. I've used int, and it returns 0 if
+ *  it fails, otherwise 1
+ */
+int XPutPixel(XImage *ximage, int x, int y, unsigned long pixel)
+{
+	int pos = 0;
+	unsigned long val = 0;
+	
+	/* check ximage points to an image */
+	if (ximage == NULL)
+	{
+		return 0;
+	}
+	
+	/* check x & y are within range */
+	if ((x < 0 || x > ximage->width - 1) ||
+	    (y < 0 || y > ximage->height - 1))
+	{
+		return 0;
+	}
+
+	/* only support images of bit depth 1 or > 8 */
+	if (ximage->depth == 1)
+	{
+		int byte_pos = 0;
+		int oldval = 0;
+		int i = 0;
+		
+		pos = (ximage->bytes_per_line * y) + (x / 8);
+		byte_pos = x % 8;
+		oldval = ximage->data[pos];
+		for (i=0; i<8; i++)
+		{
+			val <<= 1;
+			val = ((val & 0xFE) + ((byte_pos != i) ? oldval >> i : pixel)) & 0x01;
+		}
+	}
+	else
+	{
+		pos = (ximage->width * y) + x;
+		val = pixel;
+	}
+
+	ximage->data[pos] = val;
+
+	return 1;
+}
+
 /*-
  *  XQueryColor
  *    not currently used
@@ -2781,6 +2850,13 @@ void XUnmapWindow(Display *display, Window w)
 {
 }
 
+void glXSwapBuffers(Display *display, Window w)
+{
+	SwapBuffers(hdc);
+}
+
+
+
 #endif /* WIN32 */
 
 /* -------------------------------------------------------------------- */
