@@ -34,7 +34,7 @@ extern HDC hdc;                           /* device context */
 extern int cred, cgreen, cblue;           /* color reference of the pen */
 extern unsigned char *red, *green, *blue; /* holds a list of colors */
 extern int colorcount;                    /* number of colors used */
-unsigned int randommode;                  /* number of mode to index */
+extern unsigned int randommode;           /* number of mode to index */
 extern RECT rc;                           /* coords of the screen */
 
 /*-
@@ -973,6 +973,8 @@ void XUngrabKeyboard(Display *display, Time time);
 void XUngrabPointer(Display *display, Time time);
 void XUngrabServer(Display *display);
 void XUnmapWindow(Display *display, Window w);
+
+void glXSwapBuffers(Display *display, Window w);
 /* -------------------------------------------------------------------- */
 
 #if defined(__cplusplus) || defined(c_plusplus)
