@@ -330,6 +330,7 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 			sizeof (struct dirent *))) == NULL) {
 		if (debug)
 			(void) printf("scan_dir no memory\n");
+		(void) closedir(dirp);
 		return (-1);
 	}
 	num_list_tmp = 0;
@@ -347,6 +348,7 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 					sizeof (struct dirent *))) == NULL) {
 				if (debug)
 					(void) printf("scan_dir no memory\n");
+				(void) closedir(dirp);
 				return (-1);
 			}
 		}
@@ -355,8 +357,10 @@ scan_dir(const char *directoryname, struct dirent ***namelist,
 #ifdef SVR4
 			 + strlen(new_entry->d_name)
 #endif
-		)) == NULL)
+		)) == NULL) {
+				(void) closedir(dirp);
 				return (-1);
+		}
 
 		(void) strcpy(namelist_tmp[num_list_tmp - 1]->d_name, new_entry->d_name);
 
