@@ -354,7 +354,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_invert
 	{(char *) "invert", init_invert, draw_invert, release_invert,
 	 draw_invert, init_invert, (ModeHook *) NULL, &invert_opts,
-	 100, 1, 1, 1, 64, 1.0, (char *) "",
+	 80000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a sphere inverted without wrinkles", 0, NULL},
 #endif
 #ifdef MODE_juggle
@@ -363,6 +363,12 @@ LockStruct  LockProcs[] =
 	 10000, 200, 1000, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a Juggler, juggling", 0, NULL},
 #endif
+#ifdef MODE_juggler3d
+	{(char *) "juggler3d", init_juggler3d, draw_juggler3d, release_juggler3d,
+	 draw_juggler3d, change_juggler3d, (ModeHook *) NULL, &juggler3d_opts,
+	 10000, 200, 1000, 1, 64, 1.0, (char *) "",
+	 (char *) "Shows a 3D Juggler, juggling", 0, NULL},
+#endif
 #ifdef MODE_julia
 	{(char *) "julia", init_julia, draw_julia, release_julia,
 	 refresh_julia, init_julia, (ModeHook *) NULL, &julia_opts,
@@ -408,7 +414,7 @@ LockStruct  LockProcs[] =
 #ifdef MODE_life1d
 	{(char *) "life1d", init_life1d, draw_life1d, release_life1d,
 	 refresh_life1d, init_life1d, (ModeHook *) NULL, &life1d_opts,
-	 10000, 1, 10, 0, 64, 1.0, (char *) "",
+	 100000, -5, 1200, -15, 64, 1.0, (char *) "",
 	 (char *) "Shows Wolfram's game of 1D Life", 0, NULL},
 #endif
 #ifdef MODE_life3d
