@@ -25,4 +25,5 @@
 #define RasterNoMemory    -3
 #define RasterColorFailed -4
 
-extern int  RasterFileToImage(ModeInfo * mi, char *filename, XImage ** image);
+extern int  RasterFileToImage(ModeInfo * mi, char *filename, XImage ** image ,
+			      Colormap m_cmap );
