@@ -77,7 +77,10 @@ static LockStruct LockProcs[] =
 #if defined( USE_GL ) && defined( HAVE_CXX )
 {(char *) "invert", (char *) "Shows a sphere inverted without wrinkles"},
 #endif
-{(char *) "juggle", (char *) "Shows a Juggler, juggling"},
+{(char *) "juggle", (char *) "(null)"},
+#ifdef USE_GL
+{(char *) "juggler3d", (char *) "Shows a 3D Juggler, juggling"},
+#endif
 {(char *) "julia", (char *) "Shows the Julia set"},
 {(char *) "kaleid", (char *) "Shows a kaleidoscope"},
 {(char *) "kumppa", (char *) "Shows kumppa"},
@@ -139,7 +142,7 @@ static LockStruct LockProcs[] =
 #ifdef USE_GL
 {(char *) "sierpinski3d", (char *) "Shows GL's Sierpinski gasket"},
 #endif
-#if defined(USE_GL) && defined( USE_UNSTABLE )
+#ifdef USE_GL
 {(char *) "skewb", (char *) "Shows an auto-solving Skewb"},
 #endif
 {(char *) "slip", (char *) "Shows slipping blits"},
